/*
 *  User Event - soundman functions
 *  Copyright (c) by Jaroslav Kysela <perex@jcu.cz>
 */

#define __KERNEL_SYSCALLS__
#include "driver.h"
#include <linux/unistd.h>

extern char snd_soundman_path[];

#if 1
#define SND_SOUNDMAN_IS_BROKEN
#endif

#ifndef SND_SOUNDMAN_IS_BROKEN

static char snd_user_event_cardnum[16] = "global";
static char snd_user_event_event[16] = "none";
static char *snd_user_event_argv[] = { "soundman", snd_user_event_cardnum, snd_user_event_event, NULL, };
static char *snd_user_event_envp[] = { "HOME=/", "TERM=linux", NULL, };
static int errno;

snd_mutex_define_static( soundman );

int snd_user_event_thread( void *data )
{
  mm_segment_t fs;
  int fd, err = 0;
  
  errno = 0;
  fs = get_fs();
  set_fs( get_ds() );		/* ok.. emulate user space in kernel space */
  close(0); close(1); close(2);
  setsid();
  snd_printk( "errno = %i\n", errno );
#if 0
  (void) sys_open( "/dev/tty11", O_RDWR, 0 );
  snd_printk( "X - errno = %i\n", errno );
#endif
#ifdef SNDCFG_DEBUG
  (void) open( "/dev/tty11", O_RDWR, 0 );
  snd_printk( "errno = %i\n", errno );
  (void) dup( 0 );
  snd_printk( "errno = %i\n", errno );
  (void) dup( 0 );
                 /* 01234567890123456789012345678901 */
  snd_printk( "errno = %i\n", errno );
  (void) write( 0, "sound man thread initialized ok\n", 32 );
  snd_printk( "errno = %i\n", errno );
#endif
  if ( (fd = open( "/etc/hosts", O_RDONLY, 0 )) < 0 ) {
    set_fs( fs );
    snd_printk( "failed !!! errno = %i\n", errno );
    return 0;
  }
  close( fd );
  if ( (fd = open( snd_soundman_path, O_RDONLY, 0 )) < 0 ) {
    set_fs( fs );
    return 0;			/* not found, but it's probably ok */
  }
  close( fd );
  if ( execve( snd_soundman_path, snd_user_event_argv, snd_user_event_envp ) < 0 )
    err = -errno;
  snd_printk( "failed to load %s for user event %s (%i)\n", snd_soundman_path, snd_user_event_event, err );
  set_fs( fs );
  return err;
}

int snd_user_event( snd_card_t *card, char *event )
{
  int pid, status;

  if ( !event || *event == '\0' ) return -EINVAL;
  snd_mutex_down_static( soundman );
  memset( snd_user_event_cardnum, 0, sizeof( snd_user_event_cardnum ) );
  memset( snd_user_event_event, 0, sizeof( snd_user_event_event ) );
  if ( !card )
    strcpy( snd_user_event_cardnum, "global" );
   else
    sprintf( snd_user_event_cardnum, "%i", card -> number );
  strncpy( snd_user_event_event, event, sizeof( snd_user_event_event ) - 1 );
  pid = kernel_thread( snd_user_event_thread, NULL, SIGCHLD );
  if ( pid < 0 ) {
    snd_printk( "fork failed for user event" );
  }
  waitpid( pid, &status, 0 );
  snd_mutex_up_static( soundman );
  return 0;
}

#else

int snd_user_event( snd_card_t *card, char *event )
{
  return 0;
}

#endif
